﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.ProjectManagement.BusinessEntities;
using TaskViewer.DataService.DataAccess.Generic;
using System.Globalization;

namespace TaskViewer.DataService.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a SyncTask
    /// and generating the corresponding SQL to insert that
    /// SyncTask into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// SyncTask object with it.
    /// </summary>
    internal class SyncTaskInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<SyncTask>
    {
        /// <summary>
        /// Creates the SyncTaskInsertFactory to build an insert statement for
        /// the given SyncTask object.
        /// </summary>
        /// <param name="SyncTask">New SyncTask to insert into the database.</param>
        public SyncTaskInsertFactory()
        {
        }

        #region IInsertFactory<SyncTask> Members

        public DbCommand ConstructInsertCommand(Database db, SyncTask syncTask)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.InsertTask");

            db.AddInParameter(command, "AssignedTo", DbType.Int32, syncTask.AssignedTo);
            db.AddInParameter(command, "DateDue", DbType.DateTime, syncTask.DateDue);
            db.AddInParameter(command, "ModifiedBy", DbType.Int32, syncTask.ModifiedBy);
            db.AddInParameter(command, "Progress", DbType.Int32, syncTask.Progress);
            db.AddInParameter(command, "PriorityID", DbType.Int32, syncTask.PriorityID);
            db.AddInParameter(command, "ProjectID", DbType.Int32, syncTask.ProjectID);
            if (syncTask.TaskDescription != null)
            {
                db.AddInParameter(command, "TaskDescription", DbType.String, syncTask.TaskDescription);
            }
            if (syncTask.TaskSummary != null)
            {
                db.AddInParameter(command, "TaskSummary", DbType.String, syncTask.TaskSummary);
            }
            db.AddInParameter(command, "StatusID", DbType.Int32, syncTask.StatusID);
            return command;
        }

        public void SetNewID(Database db, DbCommand command, SyncTask syncTask)
        {
            //TODO: Provide set mapping
        }

        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "AssignedTo":
                    return "AssignedTo";
                case "DateDue":
                    return "DateDue";
                case "ModifiedBy":
                    return "ModifiedBy";
                case "Progress":
                    return "Progress";
                case "PriorityID":
                    return "PriorityID";
                case "ProjectID":
                    return "ProjectID";
                case "TaskDescription":
                    return "TaskDescription";
                case "TaskSummary":
                    return "TaskSummary";
                case "StatusID":
                    return "StatusID";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }

}

