﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.ProjectManagement.BusinessEntities;
using TaskViewer.DataService.DataAccess.Generic;
using System.Globalization;

namespace TaskViewer.DataService.DataAccess
{
    /// <summary>
    /// This factory class takes a SyncTask object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class SyncTaskUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<SyncTask>
    {
        /// <summary>
        /// Creates the SyncTaskUpdateFactory to build an update statement for
        /// the given SyncTask object.
        /// </summary>
        /// <param name="SyncTask">SyncTask to update into the database.</param>
        public SyncTaskUpdateFactory()
        {
        }
        
        #region IUpdateFactory<SyncTask> Members

        public DbCommand ConstructUpdateCommand(Database db, SyncTask syncTask)
        {
			DbCommand command = db.GetStoredProcCommand("dbo.UpdateTask");
			
			db.AddInParameter(command, "AssignedTo", DbType.Int32, syncTask.AssignedTo);
			db.AddInParameter(command, "DateCreated", DbType.DateTime, syncTask.DateCreated);
			db.AddInParameter(command, "DateDue", DbType.DateTime, syncTask.DateDue);
			db.AddInParameter(command, "DateModified", DbType.DateTime, syncTask.DateModified);
			db.AddInParameter(command, "IsDeleted", DbType.Boolean, syncTask.IsDeleted);
			db.AddInParameter(command, "ModifiedBy", DbType.Int32, syncTask.ModifiedBy);
			db.AddInParameter(command, "PriorityID", DbType.Int32, syncTask.PriorityID);
			db.AddInParameter(command, "Progress", DbType.Int32, syncTask.Progress);
			db.AddInParameter(command, "ProjectID", DbType.Int32, syncTask.ProjectID);
			db.AddInParameter(command, "StatusID", DbType.Int32, syncTask.StatusID);
			if(syncTask.TaskDescription != null)
			{
				db.AddInParameter(command, "TaskDescription", DbType.String, syncTask.TaskDescription);
			}
			db.AddInParameter(command, "TaskID", DbType.Int32, syncTask.TaskID);
			if(syncTask.TaskSummary != null)
			{
				db.AddInParameter(command, "TaskSummary", DbType.String, syncTask.TaskSummary);
			}
			db.AddInParameter(command, "Original_DateModified", DbType.DateTime, syncTask.OriginalDateModified);
			db.AddInParameter(command, "Original_DateDue", DbType.DateTime, syncTask.OriginalDateDue);
			db.AddInParameter(command, "Original_DateCreated", DbType.DateTime, syncTask.OriginalDateCreated);
			db.AddInParameter(command, "Original_AssignedTo", DbType.Int32, syncTask.OriginalAssignedTo);
			db.AddInParameter(command, "Original_ModifiedBy", DbType.Int32, syncTask.OriginalModifiedBy);
			db.AddInParameter(command, "Original_IsDeleted", DbType.Boolean, syncTask.OriginalIsDeleted);
			db.AddInParameter(command, "Original_PriorityID", DbType.Int32, syncTask.OriginalPriorityID);
			db.AddInParameter(command, "Original_Progress", DbType.Int32, syncTask.OriginalProgress);
			db.AddInParameter(command, "Original_ProjectID", DbType.Int32, syncTask.OriginalProjectID);
			db.AddInParameter(command, "Original_StatusID", DbType.Int32, syncTask.OriginalStatusID);
			if(syncTask.OriginalTaskDescription != null)
			{
				db.AddInParameter(command, "Original_TaskDescription", DbType.String, syncTask.OriginalTaskDescription);
			}
			if(syncTask.OrignalTaskSummary != null)
			{
				db.AddInParameter(command, "Original_TaskSummary", DbType.String, syncTask.OrignalTaskSummary);
			}
			return command;
        }
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "AssignedTo":
					return "AssignedTo";
				case "DateCreated":
					return "DateCreated";
				case "DateDue":
					return "DateDue";
				case "DateModified":
					return "DateModified";
				case "IsDeleted":
					return "IsDeleted";
				case "ModifiedBy":
					return "ModifiedBy";
				case "PriorityID":
					return "PriorityID";
				case "Progress":
					return "Progress";
				case "ProjectID":
					return "ProjectID";
				case "StatusID":
					return "StatusID";
				case "TaskDescription":
					return "TaskDescription";
				case "TaskID":
					return "TaskID";
				case "TaskSummary":
					return "TaskSummary";
				case "Original_DateModified":
					return "OriginalDateModified";
				case "Original_DateDue":
					return "OriginalDateDue";
				case "Original_DateCreated":
					return "OriginalDateCreated";
				case "Original_AssignedTo":
					return "OriginalAssignedTo";
				case "Original_ModifiedBy":
					return "OriginalModifiedBy";
				case "Original_IsDeleted":
					return "OriginalIsDeleted";
				case "Original_PriorityID":
					return "OriginalPriorityID";
				case "Original_Progress":
					return "OriginalProgress";
				case "Original_ProjectID":
					return "OriginalProjectID";
				case "Original_StatusID":
					return "OriginalStatusID";
				case "Original_TaskDescription":
					return "OriginalTaskDescription";
				case "Original_TaskSummary":
					return "OrignalTaskSummary";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }
}

